
/*==============================================================================
 *
 * (#)Utils.java
 * created      2002/11/24
 * version   $Revision: 1.2 $Date: 2003/12/07 10:56:44 $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *==============================================================================
 */

package com.miami.romeo.config;

import java.awt.*;
import java.io.*;
import java.util.*;
import java.net.*;

import javax.swing.*;

import com.miami.romeo.main.Info;
import com.miami.romeo.internet.Hyperlink;

public class Utils {
  //******* Classes needed by Utils.java
  // Configuration files.
  private File config_file = null;
  private File language_file = null;
  // This class reads and writes configuration files
  Config config = null;
  Config config2 = null;
  // A simple browser
  Hyperlink hyperlink = null;
  Info info = null;

  // Generic
  public int x;

  // Frames size.
  private int SIZE_X;
  private int SIZE_Y;

  // Frame location.
  private int LOCATION_X;
  private int LOCATION_Y;

  /**
   * Parameters for the constructor: String config_name.
   */
  public Utils(String config_name) {
    checkFiles();
    this.config_file = new File(config_name);
    this.config = new Config(config_file);
  }

  /**
   * Parameters for the constructor: File config_file.
   */
  public Utils(File config_file) {
    checkFiles();
    this.config_file = config_file;
    this.config = new Config(config_file);
  }

  /**
   * Parameters for the constructor: String config_name, String language_name.
   */
  public Utils(String config_name, String language_name) {
    checkFiles();
    this.config_file = new File(config_name);
    this.config = new Config(config_file);
    this.language_file = new File(language_name);
    this.config2 = new Config(language_file);
  }

  /**
   * Parameters for the constructor: File config_file, File language_file.
   */
  public Utils(File config_file, File language_file) {
    checkFiles();
    this.config_file = config_file;
    this.config = new Config(config_file);
    this.language_file = language_file;
    this.config2 = new Config(language_file);
  }

  /**
   * Constructor without parameters.
   */
  public Utils() {
      checkFiles();
      // Configuration file: path.
      String config_name = System.getProperty("user.home") + File.separator + 
                           ".romeo4xmame" + File.separator + "Romeo.ini";

      // Configuration file
      this.config_file = new File(config_name);

      this.config = new Config(config_file);

      // Which language are we using?
      String language = config.get("Language");

      // Language file: path.
      String language_name = System.getProperty("user.home") + File.separator + 
                             ".romeo4xmame" + File.separator + "Languages"
                             + File.separator + language + ".ini";

      // Configuration file
      this.language_file = new File(language_name);

      this.config2 = new Config(language_file);
  }


  // *******************************************************
  // ****************** Utility Methods ********************
  // *******************************************************

  /**
   * This method verifies if all files needed by Romeo exist.
   */
  public void checkFiles() {
    File[] dirs = {new File(System.getProperty("user.home") + File.separator +
                            ".romeo4xmame"),
                   new File(System.getProperty("user.home") + File.separator +
                            ".romeo4xmame" + File.separator + "Games"),
                   new File(System.getProperty("user.home") + File.separator + 
		            ".romeo4xmame" + File.separator + "Languages")};

    for (int i = 0; i < dirs.length; i++) {
      if (!dirs[i].exists()) {
	dirs[i].mkdir();
      }
    }

    String[] path = {"/resources/Italian.ini",
                     "/resources/English.ini",
                     "/resources/Spanish.ini", // Thanks to Luis Oscar Villalvazo <luisoscar@users.sourceforge.net>
                     "/resources/LICENSE",
		     "/resources/Romeo.ini"};

    File[] files = {new File(System.getProperty("user.home") + File.separator +
                             ".romeo4xmame" + File.separator + "Languages" +
                             File.separator + "Italian.ini"),
                    new File(System.getProperty("user.home") + File.separator +
		             ".romeo4xmame" + File.separator + "Languages" +
		             File.separator + "English.ini"),
                    new File(System.getProperty("user.home") + File.separator +
		             ".romeo4xmame" + File.separator + "Languages" +
		             File.separator + "Spanish.ini"),
                    new File(System.getProperty("user.home") + File.separator +
		             ".romeo4xmame" + File.separator + "Languages" +
		             File.separator + "LICENSE"),
		    new File(System.getProperty("user.home") + File.separator + 
		             ".romeo4xmame" + File.separator + "Romeo.ini")};

    for (int i = 0; i < files.length; i++) {
      if (!files[i].exists()) {
        // open data
        URL url = getClass().getResource(path[i]);

	try {
	  // convert url to buffered string
	  InputStream is = url.openStream();
	  InputStreamReader isr = new InputStreamReader(is);
	  BufferedReader reader = new BufferedReader(isr);
	  BufferedWriter writer = new BufferedWriter(new FileWriter(files[i]));

	  // read one line at a time
	  String line = reader.readLine();
	  while(line != null) {
	    writer.write(line + "\n");

	    line = reader.readLine();
	  }

	  writer.close();
	  reader.close();
	  isr.close();
	  is.close();
	}
	catch(IOException ioe) {
	  System.err.println(ioe.getMessage());
	}
      }
    }
  }

  /**
   * This method returns a mnemonic. Typically used as
   * keyboard shortcuts in menu items.
   */
  public char getMnemonic(String key) {
    return (getString(key)).charAt(0);
  }

  /**
   * Creates an icon 16x16 from an image contained in the "images" directory.
   */
  public ImageIcon createImageIcon16x16(String filename) {
    ImageIcon icon = null;
    String path;
    int i = config.getInt("Use_iconset");

    if (i == 1)
      path = "/images/icons/16x16/" + filename;
    else if (i == 2)
      path = "/images/icons2/16x16/" + filename;
    else if (i == 3)
      path = "/images/icons3/16x16/" + filename;
    else
      path = "/images/icons/16x16/" + filename;

    icon = new ImageIcon(getClass().getResource(path));
    
    return icon;
  }

  /**
   * Creates an icon 24x24 from an image contained in the "images" directory.
   */
  public ImageIcon createImageIcon24x24(String filename) {
    ImageIcon icon = null;
    String path;
    int i = config.getInt("Use_iconset");

    if (i == 1)
      path = "/images/icons/24x24/" + filename;
    else if (i == 2)
      path = "/images/icons2/24x24/" + filename;
    else if (i == 3)
      path = "/images/icons3/24x24/" + filename;
    else
      path = "/images/icons/24x24/" + filename;

    icon = new ImageIcon(getClass().getResource(path));
    
    return icon;
  }

  /**
   * Creates an icon 32x32 from an image contained in the "images" directory.
   */
  public ImageIcon createImageIcon32x32(String filename) {
    ImageIcon icon = null;
    String path;
    int i = config.getInt("Use_iconset");

    if (i == 1)
      path = "/images/icons/32x32/" + filename;
    else if (i == 2)
      path = "/images/icons2/32x32/" + filename;
    else if (i == 3)
      path = "/images/icons3/32x32/" + filename;
    else
      path = "/images/icons/32x32/" + filename;

    icon = new ImageIcon(getClass().getResource(path));
    
    return icon;
  }

  /**
   * This method returns a string value to identify a generic component.
   */
  public String getString(String text) {
    String value = null;

    try {
      value = config2.get(text);
    }
    catch (Exception e) {
	System.err.println("Could not find value for: " + text);
    }
    if(value == null) {
      value = "Could not find resource: " + text + "  ";
    }
    return value;
  }
  
  /**
   *  his method sets the messages in a label
   */
  public void message(final JLabel label, final String str) {
    if (str != null) {
      Runnable runner = new Runnable() {
        public void run() {
          label.setText(str);
        }
      };
      SwingUtilities.invokeLater(runner);
    }
  }

  /**
   *  This method sets the progress bar value
   */
  public void setProgressBarValue(final JProgressBar progressBar,
                                  final int value) {
    Runnable runner = new Runnable() {
      public void run() {
        progressBar.setValue(value);
      }
    };
    SwingUtilities.invokeLater(runner);
  }

  /**
   *  This method sets the progress bar maximum value
   */
  public void setProgressBarMaximum(final JProgressBar progressBar,
                                    final int value) {
    Runnable runner = new Runnable() {
      public void run() {
        progressBar.setMaximum(value);
      }
    };
    SwingUtilities.invokeLater(runner);
  }

  /**
   *  This method sets the progress bar string
   */
  public void setProgressBarString(final JProgressBar progressBar,
                                   final String string) {
    Runnable runner = new Runnable() {
      public void run() {
        progressBar.setString(string);
      }
    };
    SwingUtilities.invokeLater(runner);
  }

  /**
   * This method runs system browser or hyperlink module
   * to go to an url location
   */
  public void goToUrl(String homepage) {
    config = new Config(config_file); 
    String browser = config.get("User.browser");

    if (!browser.equals("")) {
      try {
        // Start command running
        Process proc = Runtime.getRuntime().exec(browser + " " + homepage);

        // Wait for command to terminate
        try {
          proc.waitFor();
        }
        catch (InterruptedException ie) {
          System.err.println("Process was interrupted");
        }

        // Check its exit value
        if (proc.exitValue() != 0)
          System.err.println("Exit value was non-zero");
      }
      catch (IOException ioe) {
        System.err.println(ioe.getMessage());
        try {
          if(hyperlink == null)
            hyperlink = new Hyperlink(homepage);
          else
            hyperlink.setVisible(true);
        }
        catch (Exception eee) {
          System.err.println(eee.getMessage());
        }
      }
    }
    else {
      try {
        if(hyperlink == null)
          hyperlink = new Hyperlink(homepage);
        else
          hyperlink.setVisible(true);
      }
      catch (Exception ee) {
        System.err.println(ee.getMessage());
      }
    }
  }

  public String concatenateValues(String[] dir, String separator) {
    String row = new String();

    if (dir != null) {
      for (int i = 0; i < dir.length; i++) {
        row = row + dir[i] + separator;
      }

      row = row.substring(0, row.lastIndexOf(separator));
    }

    return row;
  }

  public String[] parseValues(String row, char c) {
    ArrayList str = new ArrayList();

    int idx_sm = row.indexOf(c);

    if (idx_sm > 0) {
      int last_idx_sm = row.lastIndexOf(c);

      String parsed = row.substring(0, idx_sm);
      str.add(parsed);
      String s = new String();

      // Recursively get all values
      while (idx_sm <= last_idx_sm) {
	s = row.substring(idx_sm + 1);
	int new_idx_sm = s.indexOf(c);
	if (new_idx_sm > 0) {
	  parsed = row.substring(idx_sm + 1, idx_sm + new_idx_sm + 1);
	  str.add(parsed);
	  idx_sm = idx_sm + new_idx_sm + 1;
        }
	// no more occurrences: break!
	else {
          if (s.length() > 0)
            str.add(s);
	  break;
	}
      }
    }
    else {
      if (row.length() > 0)
        str.add(row);
    }

    return (String[])str.toArray(new String[0]);
  }

  /**
   * This method retrieves an int from the configuration file
   * to be used building modules.
   */
  public int getFrameParameter(JFrame frame, String suffix) {
    try {
      // Get title frame.
      String frameTitle = frame.getTitle();

      x = config.getInt(frameTitle + suffix);

    }
    catch (Exception e) {
      System.err.println(e.getMessage());
    }
    return x;
  }

  /**
   * Save frame parameters in the configuration file.
   */
  public void saveFrameParameters(JFrame frame, Info info) {
    if (frame != null) {
      try {
        // Get title frame.
	String frameTitle = frame.getTitle();

        // *******************************************************
        // ********************   Location  **********************
        // *******************************************************

	Point l = frame.getLocationOnScreen();

	LOCATION_X = l.x;
        LOCATION_Y = l.y;

	// *******************************************************
        // ********************   Size  **************************
        // *******************************************************

	Dimension s = frame.getSize();

	SIZE_X = s.width;
	SIZE_Y = s.height;

	// Palette size
	Point p = info.palette.getLocation();

	config = new Config(config_file);
	config.add(frameTitle + "." + "LOCATION_X", LOCATION_X);
	config.add(frameTitle + "." + "LOCATION_Y", LOCATION_Y);
	config.add(frameTitle + "." + "SIZE_X", SIZE_X);
	config.add(frameTitle + "." + "SIZE_Y", SIZE_Y);
	config.add("Palette.LOCATION_X", p.x);
        config.add("Palette.LOCATION_Y", p.y);
	config.save();
      }
      catch (Exception e) {
        System.err.println(e);
      }
    }
  }
}
