
/*==============================================================================
 *
 * (#)SystemInfo.java
 * created         2003/02/07
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *==============================================================================
 */

package com.miami.romeo.main.about;

//****** imports
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import com.miami.romeo.config.*;
import com.miami.romeo.main.table.SortButtonRenderer;
import com.miami.romeo.main.table.TableSorter2;
import com.miami.romeo.main.table.SortableTableModel;

/**
 * SystemInfo.<br>
 * this class creates a panel for the about box.
 *
 * @author       $Author: Stefano Bolli
 * @version      $Revision: 1.1.1.1 $Date: 2003/10/18 03:17:51 $
 */
public class SystemInfo extends JPanel {
  Utils utils = new Utils();

  public JTabbedPane sPane = null;

  public SystemInfo() {
    final String[] names = {utils.getString("AboutBox.table_name"),
                            utils.getString("AboutBox.table_value")};

    SortableTableModel dm = new SortableTableModel() {
      public Class getColumnClass(int col) {
        return String.class;
      }
      public boolean isCellEditable(int row, int col) {
        return false;
      }
    };

    TableSorter2 sorter = new TableSorter2(dm);

    dm.setDataVector(new Object[][] {
                       {"java.version", System.getProperty("java.version")},
                       {"java.vendor", System.getProperty("java.vendor")},
                       {"java.vendor.url", System.getProperty("java.vendor.url")},
                       {"java.home", System.getProperty("java.home")},
                       {"java.vm.specification.version", System.getProperty(
                                              "java.vm.specification.version")},
                       {"java.vm.specification.name", System.getProperty(
                                                 "java.vm.specification.name")},
                       {"java.vm.version", System.getProperty(
                                                            "java.vm.version")},
                       {"java.vm.vendor", System.getProperty(
                                                             "java.vm.vendor")},
                       {"java.vm.name", System.getProperty("java.vm.name")},
                       {"java.specification.version", System.getProperty(
                                                 "java.specification.version")},
                       {"java.specification.vendor", System.getProperty(
                                                  "java.specification.vendor")},
                       {"java.specification.name", System.getProperty(
                                                    "java.specification.name")},
                       {"java.class.version", System.getProperty(
                                                         "java.class.version")},
                       {"java.class.path", System.getProperty(
                                                            "java.class.path")},
                       {"java.ext.dirs", System.getProperty("java.ext.dirs")},
                       {"os.name", System.getProperty("os.name")},
                       {"os.arch", System.getProperty("os.arch")},
                       {"os.version", System.getProperty("os.version")},
                       {"file.separator", System.getProperty(
                                                             "file.separator")},
                       {"path.separator", System.getProperty(
                                                             "path.separator")},
                       {"line.separator", System.getProperty(
                                                             "line.separator")},
                       {"user.name", System.getProperty("user.name")},
                       {"user.home", System.getProperty("user.home")},
                       {"user.dir", System.getProperty("user.dir")}}, names);

    final JTable table = new JTable(dm);
    table.setShowHorizontalLines(true);
    table.setShowVerticalLines(true);
    table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
    table.getTableHeader().setReorderingAllowed(false);
    SortButtonRenderer renderer = new SortButtonRenderer();
    TableColumnModel columnModel = table.getColumnModel();

    TableColumn column = null;

    for (int i = 0; i < 2; i++) {
      column = columnModel.getColumn(i);
      column.setHeaderRenderer(renderer);
      column.setPreferredWidth(200);
    }

    JTableHeader header = table.getTableHeader();
    header.addMouseListener(new HeaderListener(header, renderer));

    JScrollPane scrollpane = new JScrollPane(table);

    sPane = new JTabbedPane();
    sPane.addTab(utils.getString("AboutBox.system_label"), null, scrollpane);

    setLayout(new BorderLayout());
    add(sPane, BorderLayout.CENTER);
  }

  class HeaderListener extends MouseAdapter {
    JTableHeader   header;
    SortButtonRenderer renderer;

    HeaderListener(JTableHeader header, SortButtonRenderer renderer) {
      this.header   = header;
      this.renderer = renderer;
    }

    public void mousePressed(MouseEvent e) {
      int col = header.columnAtPoint(e.getPoint());
      int sortCol = header.getTable().convertColumnIndexToModel(col);
      renderer.setPressedColumn(col);
      renderer.setSelectedColumn(col);
      header.repaint();

      if (header.getTable().isEditing()) {
        header.getTable().getCellEditor().stopCellEditing();
      }

      boolean isAscent;
      if (SortButtonRenderer.DOWN == renderer.getState(col)) {
        isAscent = true;
      } else {
        isAscent = false;
      }
      ((SortableTableModel)header.getTable().getModel())
        .sortByColumn(sortCol, isAscent);
    }

    public void mouseReleased(MouseEvent e) {
      int col = header.columnAtPoint(e.getPoint());
      renderer.setPressedColumn(-1);                // clear
      header.repaint();
    }
  }
}
